"""
This module contains functionality to translate between datekeys and dates.

Technically there are tables containing this data, but I don't think
it's necessary to actually query them. We can just calculate them directly.
Should be much quicker.
"""

from datetime import datetime, timedelta

import pandas

# from connection_handling import _get_snowflake_connection


def datekey_to_date(datekey: int) -> datetime:
    """
    Convert a datekey to a datetime object.
    """
    base_date = datetime(1900, 1, 1)
    return base_date + timedelta(days=datekey - 1)


def date_to_datekey(date: datetime) -> int:
    """
    Convert a datetime object to a datekey.
    """
    base_date = datetime(1900, 1, 1)
    return (date - base_date).days + 1


def get_datekey_range(start_date: datetime, end_date: datetime) -> list[int]:
    """
    Get a list of datekeys between two dates.
    """
    return [date_to_datekey(x) for x in pandas.date_range(start_date, end_date)]


def get_date_range(start_datekey: int, end_datekey: int) -> list[datetime]:
    """
    Get a list of dates between two datekeys.
    """
    return [datekey_to_date(x) for x in range(start_datekey, end_datekey + 1)]


def monthkey_to_date(monthkey: int) -> datetime:
    """
    Convert a monthkey to a datetime object representing the first day of the month.
    """
    year = 1900 + (monthkey - 1) // 12
    month = (monthkey - 1) % 12 + 1
    return datetime(year, month, 1)


def date_to_monthkey(date: datetime) -> int:
    """
    Convert a datetime object to a monthkey.
    """
    return (date.year - 1900) * 12 + date.month


def quarterkey_to_date(quarterkey: int) -> datetime:
    """
    Convert a quarterkey to a datetime object representing the first day of the quarter.
    """
    year = 1900 + (quarterkey - 1) // 4
    quarter = (quarterkey - 1) % 4 + 1
    month = (quarter - 1) * 3 + 1
    return datetime(year, month, 1)


def date_to_quarterkey(date: datetime) -> int:
    """
    Convert a datetime object to a quarterkey.
    """
    quarter = (date.month - 1) // 3 + 1
    return (date.year - 1900) * 4 + quarter


def yearkey_to_date(yearkey: int) -> datetime:
    """
    Convert a yearkey to a datetime object representing the first day of the year.
    """
    year = 1900 + yearkey - 1
    return datetime(year, 1, 1)


def date_to_yearkey(date: datetime) -> int:
    """
    Convert a datetime object to a yearkey.
    """
    return date.year - 1900 + 1


# Commented out in-progress code
# def date_dim(
#     datekey: int | None = None, date: datetime | None = None
# ) -> pandas.DataFrame:
#     """
#     Retrieve corresponding datekey from a given date or vice versa.
#     """
#     if date and datekey:
#         raise Exception("Only one of datekey or date must be provided")
#     if not datekey and not date:
#         raise Exception("Either datekey or date must be provided")

#     conn = _get_snowflake_connection(schema="TIME_DIMENSION")
#     try:
#         if datekey:
#             query = f"SELECT * FROM DATE_DIM WHERE DATEKEY = {datekey}"
#         else:
#             query = f"SELECT * FROM DATE_DIM WHERE DATE = '{date}'"
#         cur = conn.cursor()
#         cur.execute(query)
#         df = cur.fetch_pandas_all()
#     except Exception as e:
#         raise Exception("Error reading Snowflake table", e)
#     finally:
#         conn.close()
#     return df



"""
Module contains a function for interacting with the Cantaloupe Spotlight API.
"""

import os
import shutil
from datetime import datetime
from typing import List, Tuple

import pandas
import requests
from tenacity import (
    retry,
    retry_if_exception_type,
    stop_after_attempt,
    wait_exponential,
)


@retry(
    stop=stop_after_attempt(5),
    wait=wait_exponential(multiplier=1, min=4, max=10),
    retry=retry_if_exception_type(
        (requests.exceptions.ConnectionError, requests.exceptions.Timeout)
    ),
)
def run_report(
    report_id: str, params: List[Tuple[str, str]] = None, dtype: dict = None
) -> pandas.DataFrame:
    """
    Send GET request to Cantaloupe API to run report and receive excel file data.
    Uses Basic authentication with username and password.
    Returns a pandas dataframe of the report data.

    :param report_id: The ID of the report to run.
    :param params: A list of tuples to pass as parameters in the GET request. Usually date ranges.
    :param dtype: Dictionary of column names and data types to cast columns to.
    """
    # Check for environment variables
    if "CLO_USERNAME" not in os.environ:
        raise Exception("CLO_USERNAME environment variable not set")
    if "CLO_PASSWORD" not in os.environ:
        raise Exception("CLO_PASSWORD environment variable not set")

    if params is None:
        params = []
    params.append(("ReportId", report_id))

    try:
        response = requests.get(
            os.environ.get("CLO_BASE_URL", "https://api.mycantaloupe.com")
            + "/Reports/Run",
            auth=(os.environ["CLO_USERNAME"], os.environ["CLO_PASSWORD"]),
            params=params,
            timeout=600,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not run report: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not run report: {e}")

    excel_data = response.content

    try:
        # Save temp excel file to local directory
        with open(f"report{report_id}.xlsx", "wb") as f:
            f.write(excel_data)
    except Exception as e:
        raise Exception(f"Error saving excel file {e}")

    try:
        report_df = pandas.read_excel(
            f"report{report_id}.xlsx", sheet_name="Report", dtype=dtype
        )
    except Exception as e:
        raise Exception(f"Error reading excel file: {e}")

    # Delete temp excel file
    if len(report_df) > 0:
        _handle_temp_file(report_id)

    return report_df


def _handle_temp_file(report_id: str):
    """
    Helper function that gets called after the report is run.
    """
    if os.environ.get("CLO_ARCHIVE_FILES", "false").lower() == "true":
        try:
            new_dir = os.path.join(
                os.getcwd(), "Archive", datetime.now().strftime("%Y-%m-%d")
            )
            os.makedirs(new_dir, exist_ok=True)
            shutil.move(
                f"report{report_id}.xlsx",
                os.path.join(new_dir, f"report{report_id}.xlsx"),
            )
        except Exception as e:
            print("Error moving excel file: ", e)
            raise Exception("Error moving excel file", e)
    else:
        try:
            os.remove(f"report{report_id}.xlsx")
        except Exception as e:
            print("Error deleting excel file: ", e)
            raise Exception("Error deleting excel file", e)



import os

import snowflake.connector


def _get_snowflake_connection(
    schema: str = "PUBLIC",
) -> snowflake.connector.SnowflakeConnection:
    """
    Connect to Snowflake data warehouse using environment variables. By default,
    connects to the main PUBLIC schema. Can be overridden to connect to others.
    """
    for env in [
        "SNOWFLAKE_USER",
        "SNOWFLAKE_PASSWORD",
        "SNOWFLAKE_ACCOUNT",
        "SNOWFLAKE_WAREHOUSE",
        "SNOWFLAKE_DATABASE",
    ]:
        if env not in os.environ:
            raise Exception(f"Missing required environment variable: {env}")

    conn = snowflake.connector.connect(
        user=os.environ["SNOWFLAKE_USER"],
        password=os.environ["SNOWFLAKE_PASSWORD"],
        account=os.environ["SNOWFLAKE_ACCOUNT"],
        warehouse=os.environ["SNOWFLAKE_WAREHOUSE"],
        database=os.environ["SNOWFLAKE_DATABASE"],
        schema=schema,
    )
    return conn



from datetime import datetime

import pandas
from clope.snow.connection_handling import _get_snowflake_connection


def get_cashless_vending_transaction_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    item: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact is about the money related to a transaction.
    Pertains to vending and micromarkets lines of business only.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param item: The item key
    :param date_range: The date range the item was sold, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM CASHLESSVENDINGTRANSACTIONFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if date_range:
            conditions.append(
                f"TRANSACTIONDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_collection_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    micro_market: int = None,
    route: int = None,
    line_of_business: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact is about the collection of money at a vend visit.
    Pertains to vending and micromarkets lines of business only.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param micro_market: The micro market key
    :param route: The route key
    :param line_of_business: The line of business key
    :param date_range: The date range the item was collected, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM COLLECTIONFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if route:
            conditions.append(f"ROUTEKEY = {route}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if date_range:
            conditions.append(
                f"VISITDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_micromarket_salesfact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    micro_market: int = None,
    item: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact is about the sales of items at a micromarket.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param micro_market: The micro market key
    :param item: The item key
    :param date_range: The date range the item was sold, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM MICROMARKETSALESFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if date_range:
            conditions.append(
                f"TRANSACTIONDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_order_fulfillment_delivery_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    line_of_business: int = None,
    machine: int = None,
    item: int = None,
    item_pack: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact tracks orders from the intial creation to delivery
    specifically providing details on order quantities at the time of order
    creation, prepick, pick, and delivery.
    PrepickQuantity reflects any changes made to the order after its initial
    creation but before picking.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param line_of_business: The line of business key
    :param machine: The machine key
    :param item: The item key
    :param item_pack: The item pack key
    :param date_range: The date range the item was delivered, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM ORDERFULFILLMENTDELIVERYFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if date_range:
            conditions.append(
                f"ORDERDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_order_fulfillment_vending_market_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    line_of_business: int = None,
    micro_market: int = None,
    item: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact illustrates the flow of product from prepick to delivery
    for each vending machine and market section.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param line_of_business: The line of business key
    :param micro_market: The micro market key
    :param item: The item key
    :param date_range: The date range the item was delivered, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM ORDERFULFILLMENTVENDINGMARKETFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if date_range:
            conditions.append(
                f"TARGETDELIVERYDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_delivery_order_receipt_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    item: int = None,
    item_pack: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This face returns information on the receipt of a delivery order and
    the original order details.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param item: The item key
    :param item_pack: The item pack key
    :param date_range: The date range the item was received, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM RECEIVEDELIVERYORDERFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if date_range:
            conditions.append(
                f"ORDERDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_sales_revenue_by_day_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    item: int = None,
    item_pack: int = None,
    route: int = None,
    line_of_business: int = None,
    micro_market: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    First version of this fact. Provides sales revenue by day.
    For delivery, this is revenue from deliveries made.
    For vending, this is sales and revenue since the prior visit.
    For markets, this is sales revenue recorded from the market for the day.

    Note that for Markets, the Spoils and Commissions are not available in
    this view, as they are tied to the visit.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param item: The item key
    :param item_pack: The item pack key
    :param route: The route key
    :param line_of_business: The line of business key
    :param micro_market: The micro market key
    :param date_range: The date range of sales, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM RECOGNIZESALESREVENUEFACTBYDAY_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if route:
            conditions.append(f"ROUTEKEY = {route}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if date_range:
            conditions.append(
                f"VISITDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_sales_revenue_by_visit_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    item: int = None,
    item_pack: int = None,
    route: int = None,
    line_of_business: int = None,
    micro_market: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact provides sales revenue by visit.
    For delivery, this is revenue for the deliveries made.
    For vending and market, this is sales and revenue since the prior visit.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param item: The item key
    :param item_pack: The item pack key
    :param route: The route key
    :param line_of_business: The line of business key
    :param micro_market: The micro market key
    :param date_range: The date range of visits, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM RECOGNIZESALESREVENUEFACTBYVISIT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if route:
            conditions.append(f"ROUTEKEY = {route}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if date_range:
            conditions.append(
                f"VISITDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_sales_by_coil(
    branch: int = None,
    customer: int = None,
    location: int = None,
    line_of_business: int = None,
    machine: int = None,
    item: int = None,
    micro_market: int = None,
    coil: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact provides sales by coil for each day.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param line_of_business: The line of business key
    :param machine: The machine key
    :param item: The item key
    :param micro_market: The micro market key
    :param coil: The coil key
    :param date_range: The date range of sales, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM SALESBYCOILFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if coil:
            conditions.append(f"COILKEY = {coil}")
        if date_range:
            conditions.append(
                f"EXTRACTDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_scheduling_machine_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    line_of_business: int = None,
    machine: int = None,
    micro_market: int = None,
    route: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact provides information on the scheduling of machines, what caused
    it to be scheduled, whether the schedule was edited.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param line_of_business: The line of business key
    :param machine: The machine key
    :param micro_market: The micro market key
    :param route: The route key
    :param date_range: The date range the item was scheduled, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM SCHEDULINGMACHINEFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if route:
            conditions.append(f"ROUTEKEY = {route}")
        if date_range:
            conditions.append(
                f"SCHEDULEDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_scheduling_route_summary_fact(
    branch: int = None,
    route: int = None,
    line_of_business: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact is a roll-up view of the statistics associated with schedules.

    :param branch: The branch key
    :param route: The route key
    :param line_of_business: The line of business key
    :param date_range: The date range the item was scheduled, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM SCHEDULINGROUTESUMMARYFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if route:
            conditions.append(f"ROUTEKEY = {route}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if date_range:
            conditions.append(
                f"SCHEDULINGDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_telemetry_sales_fact(
    location: int = None,
    machine: int = None,
    device: int = None,
    item: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact shows the sales reported on each product at each call from a
    telemeter from a vending machine.  All products associated with the
    machine at the time of the call are included.

    :param location: The location key
    :param machine: The machine key
    :param device: The device key
    :param item: The item key
    :param date_range: The date range the item was sold, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM TELEMETRYSALESFACT_V"
        conditions = []
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if device:
            conditions.append(f"DEVICEKEY = {device}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if date_range:
            conditions.append(
                f"TRANSACTIONDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_vending_micromarket_visit_item_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    route: int = None,
    line_of_business: int = None,
    micro_market: int = None,
    item: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact provides the item level inventory and delivery information
    for replenishment visits for Vending and Micromarkets.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param route: The route key
    :param line_of_business: The line of business key
    :param micro_market: The micro market key
    :param item: The item key
    :param date_range: The date range the item was visited, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM VENDINGMICROMARKETVISITITEMFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if route:
            conditions.append(f"ROUTEKEY = {route}")
        if line_of_business:
            conditions.append(f"LINEOFBUSINESSKEY = {line_of_business}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if date_range:
            conditions.append(
                f"VISITDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_warehouse_inventory_fact(
    branch: int = None,
    warehouse: int = None,
    item: int = None,
    item_pack: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact provides a view of the start of day inventory for each warehouse.

    :param branch: The branch key
    :param warehouse: The warehouse key
    :param item: The item key
    :param item_pack: The item pack key
    :param date_range: The date range the item was received, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM WAREHOUSEINVENTORYFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if warehouse:
            conditions.append(f"WAREHOUSEKEY = {warehouse}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"PARPACKKEY = {item_pack}")
        if date_range:
            conditions.append(
                f"INVENTORYDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_warehouse_observed_inventory_fact(
    branch: int = None,
    warehouse: int = None,
    item: int = None,
    item_pack: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact provides the view of the observed inventories that were captured.

    :param branch: The branch key
    :param warehouse: The warehouse key
    :param item: The item key
    :param item_pack: The item pack key
    :param date_range: The date range the item was observed, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM WAREHOUSEOBSERVEDINVENTORYFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if warehouse:
            conditions.append(f"WAREHOUSEKEY = {warehouse}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if date_range:
            conditions.append(
                f"EFFECTIVEDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_warehouse_prod_movement_fact(
    branch: int = None,
    from_warehouse: int = None,
    to_warehouse: int = None,
    item: int = None,
    item_pack: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact is for reporting on all product movements.

    :param branch: The branch key
    :param from_warehouse: The warehouse key the product is moving from
    :param to_warehouse: The warehouse key the product is moving to
    :param item: The item key
    :param item_pack: The item pack key
    :param date_range: The date range the item was moved, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM WAREHOUSEPRODUCTMOVEMENTFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if from_warehouse:
            conditions.append(f"FROMWAREHOUSEKEY = {from_warehouse}")
        if to_warehouse:
            conditions.append(f"TOWAREHOUSEKEY = {to_warehouse}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if date_range:
            conditions.append(
                f"EFFECTIVEDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_warehouse_purchase_fact(
    branch: int = None,
    warehouse: int = None,
    item: int = None,
    item_pack: int = None,
    supplier: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact provides the view of the purchases made by the warehouse.

    :param branch: The branch key
    :param warehouse: The warehouse key
    :param item: The item key
    :param item_pack: The item pack key
    :param supplier: The supplier key
    :param date_range: The date range the item was purchased, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM WAREHOUSEPURCHASEFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if warehouse:
            conditions.append(f"WAREHOUSEKEY = {warehouse}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if supplier:
            conditions.append(f"SUPPLIERKEY = {supplier}")
        if date_range:
            conditions.append(
                f"EFFECTIVEDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_warehouse_receive_fact(
    branch: int = None,
    warehouse: int = None,
    item: int = None,
    item_pack: int = None,
    supplier: int = None,
    date_range: tuple[int, int] = None,
) -> pandas.DataFrame:
    """
    This fact provides info about what was received by the warehouse.

    :param branch: The branch key
    :param warehouse: The warehouse key
    :param item: The item key
    :param item_pack: The item pack key
    :param supplier: The supplier key
    :param date_range: The date range the item was received, as a tuple of date keys
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM WAREHOUSERECEIVEFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if warehouse:
            conditions.append(f"WAREHOUSEKEY = {warehouse}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if supplier:
            conditions.append(f"SUPPLIERKEY = {supplier}")
        if date_range:
            conditions.append(
                f"EFFECTIVEDATEKEY BETWEEN {date_range[0]} AND {date_range[1]}"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_machine_alerts_fact(
    branch: int = None,
    customer: int = None,
    location: int = None,
    machine: int = None,
    micro_market: int = None,
    effective_date_range: tuple[int, int] = None,
    added_date_range: tuple[datetime, datetime] = None,
) -> pandas.DataFrame:
    """
    This fact contains info on machine alerts that Cantaloupe has raised.
    Out of order, not dexing, etc.

    :param branch: The branch key
    :param customer: The customer key
    :param location: The location key
    :param machine: The machine key
    :param micro_market: The micro market key
    :param effective_date_range: The date range the alert was effective, as a tuple of date keys
    :param added_date_range: The date range the alert was added, as a tuple of datetime objects
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM MACHINEALERTSFACT_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if micro_market:
            conditions.append(f"MICROMARKETKEY = {micro_market}")
        if effective_date_range:
            conditions.append(
                f"CREATEDDATEKEY BETWEEN {effective_date_range[0]} AND {effective_date_range[1]}"
            )
        if added_date_range:
            conditions.append(
                f"INSERTDATETIME BETWEEN '{added_date_range[0]}' AND '{added_date_range[1]}'"
            )
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df



import sys
from datetime import datetime, timedelta

sys.path.insert(0, "")

from clope import get_machine_alerts_fact, run_report
from clope.snow.dates import date_to_datekey, datekey_to_date
from dotenv import load_dotenv

load_dotenv()

# alerts = get_machine_alerts_fact(
#     effective_date_range=(
#         date_to_datekey(datetime.now() - timedelta(days=1)),
#         date_to_datekey(datetime.now()),
#     ),
#     added_date_range=(
#         datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)
#         - timedelta(days=1),
#         datetime.now(),
#     ),
# )

df = run_report(
    "24128", [("filter3", "2024-09-06T00:00:00Z"), ("filter3", "2024-09-06T23:59:59Z")]
)

pass



"""
clope is a package for pulling data from the Cantaloupe/Seed Office system.
Primarily via the Spotlight API.
"""

from .snow.dates import *
from .snow.dimensions import *
from .snow.facts import *
from .spotlight.spotlight import run_report

if __name__ == "__main__":
    """
    Example usage
    """
    pass



import pandas
from clope.snow.connection_handling import _get_snowflake_connection


def get_operators() -> pandas.DataFrame:
    """
    Get list of Seed databases an operator uses. For most, will be 1.
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMOPERATOR_V"
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_lines_of_business() -> pandas.DataFrame:
    """
    Reference table for the three lines of business.
    Delivery, Micromarket, and Vending
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMLINEOFBUSINESS_V"
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_branches() -> pandas.DataFrame:
    """
    Get list of branches.
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMBRANCH_V WHERE BRANCHID != -1"
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_routes(branch: int = None) -> pandas.DataFrame:
    """
    Get list of routes.

    :param branch: Filter by branch key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMROUTE_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_customers(current: bool = False, branch: int = None) -> pandas.DataFrame:
    """
    Get list of customers.
    Implements SCD Type 2, so use current=True to get only current rows of
    information and filter out historical data.

    :param current: Filter by current row indicator
    :param branch: Filter by branch key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMCUSTOMER_V"
        conditions = []
        if current:
            conditions.append("CURRENTROWINDICATOR = 'Current'")
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_locations(current: bool = False, customer: int = None) -> pandas.DataFrame:
    """
    Get list of locations.
    Implements SCD Type 2, so use current=True to get only current rows of
    information and filter out historical data.

    :param current: Filter by current row indicator
    :param customer: Filter by customer key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMLOCATION_V"
        conditions = []
        if current:
            conditions.append("CURRENTROWINDICATOR = 'Current'")
        if customer:
            conditions.append(f"CUSTOMERKEY = {customer}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_machines(
    current: bool = False, location: int = None, route: int = None
) -> pandas.DataFrame:
    """
    Get list of machines.
    Implements SCD Type 2, so use current=True to get only current rows of
    information and filter out historical data.

    :param current: Filter by current row indicator
    :param location: Filter by location key
    :param route: Filter by route key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMMACHINE_V"
        conditions = []
        if current:
            conditions.append("CURRENTROWINDICATOR = 'Current'")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if route:
            conditions.append(f"ROUTEKEY = {route}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_coils(
    current: bool = False,
    machine: int = None,
    item: int = None,
) -> pandas.DataFrame:
    """
    Get list of coils. I.E. every coil in every machine planogram.
    Quite a lot of data, but tells you which product is where.
    Implements SCD Type 2, so use current=True to get only current rows of
    information and filter out historical data.

    :param current: Filter by current row indicator
    :param machine: Filter by machine key
    :param item: Filter by item key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMMACHINEPLANOGRAMCOIL_V"
        conditions = []
        if current:
            conditions.append("CURRENTROWINDICATOR = 'Current'")
        if machine:
            conditions.append(f"MACHINEKEY = {machine}")
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_micromarkets(
    current: bool = False,
    location: int = None,
    route: int = None,
) -> pandas.DataFrame:
    """
    Get list of micromarkets.
    Implements SCD Type 2, so use current=True to get only current rows of
    information and filter out historical data.

    :param current: Filter by current row indicator
    :param location: Filter by location key
    :param route: Filter by route key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMMICROMARKET_V"
        conditions = []
        if current:
            conditions.append("CURRENTROWINDICATOR = 'Current'")
        if location:
            conditions.append(f"LOCATIONKEY = {location}")
        if route:
            conditions.append(f"ROUTEKEY = {route}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_telemetry_devices() -> pandas.DataFrame:
    """
    Get list of telemetry devices.
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMDEVICE_V"
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_items(current: bool = False) -> pandas.DataFrame:
    """
    Get list of items.
    Implements SCD Type 2, so use current=True to get only current rows of
    information and filter out historical data.
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMITEM_V"
        conditions = []
        if current:
            conditions.append("CURRENTROWINDICATOR = 'Current'")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_item_packs(item: int = None) -> pandas.DataFrame:
    """
    Get list of item packs.

    :param item: Filter by item key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMITEMPACK_V"
        conditions = []
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_item_pack_barcodes(item: int = None, item_pack: int = None) -> pandas.DataFrame:
    """
    Get list of item pack barcodes.

    :param item: Filter by item key
    :param item_pack: Filter by item pack key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMITEMPACKBARCODE_V"
        conditions = []
        if item:
            conditions.append(f"ITEMKEY = {item}")
        if item_pack:
            conditions.append(f"ITEMPACKKEY = {item_pack}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_suppliers() -> pandas.DataFrame:
    """
    Get list of suppliers.
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMSUPPLIER_V"
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_supplier_branch() -> pandas.DataFrame:
    """
    Get list of supplier branches.
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMSUPPLIERBRANCH_V"
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_supplier_items() -> pandas.DataFrame:
    """
    Get list of supplier items.
    NOTE: Doesn't seem to be used yet. No rows as of writing.
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMSUPPLIERITEM_V"
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df


def get_warehouses(branch: int = None) -> pandas.DataFrame:
    """
    Get list of warehouses.

    :param branch: Filter by branch key
    """
    conn = _get_snowflake_connection()
    try:
        query = f"SELECT * FROM DIMWAREHOUSE_V"
        conditions = []
        if branch:
            conditions.append(f"BRANCHKEY = {branch}")
        if conditions:
            query += " WHERE " + " AND ".join(conditions)
        cur = conn.cursor()
        cur.execute(query)
        df = cur.fetch_pandas_all()
    except Exception as e:
        raise Exception("Error reading Snowflake table", e)
    finally:
        conn.close()
    return df



"""
TODO: This module contains functionlaity to interact with the various
table functions that have been defined in the Snowflake database. They largely
seem to be wrappers around querying the database for a particular time period
so may not be necessary to implement.
"""
